package com.gic.store.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.store.dao.mapper.TabStoreFieldMapper;
import com.gic.store.dto.StoreFieldDTO;
import com.gic.store.entity.TabStoreField;
import com.gic.store.service.StoreFieldService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/1 9:13 AM
 */
@Service("storeFieldService")
public class StoreFieldServiceImpl implements StoreFieldService{
    @Autowired
    private TabStoreFieldMapper tabStoreFieldMapper;
    @Override
    public int save(StoreFieldDTO storeFieldDTO) {
        return tabStoreFieldMapper.insert(EntityUtil.changeEntityNew(TabStoreField.class, storeFieldDTO));
    }

    @Override
    public int edit(StoreFieldDTO storeFieldDTO) {
        return tabStoreFieldMapper.updateByPrimaryKeySelective(EntityUtil.changeEntityNew(TabStoreField.class, storeFieldDTO));
    }

    @Override
    public boolean isStoreFieldCodeRepeat(String storeFieldCode, int enterpriseId) {
        return false;
    }

    @Override
    public TabStoreField getNextSort(int enterpriseId, double sort) {
        return tabStoreFieldMapper.selectNextByEnterpriseIdAndSort(enterpriseId, sort);
    }

    @Override
    public TabStoreField getPrevSort(int enterpriseId, double sort) {
        return tabStoreFieldMapper.selectPrevByStoreFieldIdAndSort(enterpriseId, sort);
    }

    @Override
    public TabStoreField getStoreFieldById(int storeFieldId) {
        return tabStoreFieldMapper.selectByPrimaryKey(storeFieldId);
    }

    @Override
    public int updateSortById(int storeFieldId, double sort) {
        TabStoreField storeField = new TabStoreField();
        storeField.setStoreFieldId(storeFieldId);
        storeField.setSort(sort);
        tabStoreFieldMapper.updateByPrimaryKeySelective(storeField);
        return 1;
    }

    @Override
    public int deleteAll(int enterpriseId) {
        return tabStoreFieldMapper.deleteAll(enterpriseId);
    }

    @Override
    public int deleteBatch(List<Integer> storeFieldIds) {
        return tabStoreFieldMapper.deleteBatch(storeFieldIds);
    }

    @Override
    public void setStoreFieldSort(int storeFieldId, int enterpriseId, int setValue) {
        if (setValue <= 1) {
            // 相当于置顶
            sortTopping(enterpriseId, storeFieldId);
        } else {
            int start = setValue - 2;
            int size = 2;

            List<TabStoreField> list = tabStoreFieldMapper.listTwoElementExceptId(storeFieldId, start, size, enterpriseId);
            if (CollectionUtils.isEmpty(list)) {
                // 置底
                sortBottom(enterpriseId, storeFieldId);
            } else {
                Double sort;
                if (list.size() == 1) {
                    // 置底
                    sort = list.get(0).getSort() + 1d;
                } else {
                    sort = (list.get(0).getSort() + list.get(1).getSort()) / 2d;
                }
                TabStoreField storeField = new TabStoreField();
                storeField.setStoreFieldId(storeFieldId);
                storeField.setSort(sort);
                tabStoreFieldMapper.updateByPrimaryKeySelective(storeField);
            }
        }
    }

    @Override
    public int delete(int storeFieldId) {
        TabStoreField storeField = new TabStoreField();
        storeField.setStoreFieldId(storeFieldId);
        storeField.setStatus(0);
        return tabStoreFieldMapper.updateByPrimaryKeySelective(storeField);
    }

    @Override
    public Page<TabStoreField> listStoreField(int enterpriseId, String search, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<TabStoreField> list = tabStoreFieldMapper.listStoreField(enterpriseId, search);
        return (Page<TabStoreField>) list;
    }

    private void sortTopping(int enterpriseId, int storeFieldId) {
        Double sort = tabStoreFieldMapper.getMinSort(enterpriseId);
        TabStoreField storeField = new TabStoreField();
        storeField.setStoreFieldId(storeFieldId);
        storeField.setSort(sort - 1d);
        tabStoreFieldMapper.updateByPrimaryKeySelective(storeField);
    }

    private void sortBottom(int enterpriseId, int storeFieldId) {
        Double sort = tabStoreFieldMapper.getMaxSort(enterpriseId);
        TabStoreField storeField = new TabStoreField();
        storeField.setStoreFieldId(storeFieldId);
        storeField.setSort(sort + 1d);
        tabStoreFieldMapper.updateByPrimaryKeySelective(storeField);
    }
}
